MODULE obcdta
   !!==============================================================================
   !!                            ***  MODULE obcdta  ***
   !! Open boundary data : read the data for the open boundaries.
   !!==============================================================================
#if defined key_obc
   !!------------------------------------------------------------------------------
   !!   'key_obc'         :                                Open Boundary Conditions
   !!------------------------------------------------------------------------------
   !!   obc_dta           : read u, v, t, s data along each open boundary
   !!   obc_dta_psi       : read psi data along each open boundary (rigid lid only)
   !!------------------------------------------------------------------------------
   !! * Modules used
   USE oce             ! ocean dynamics and tracers 
   USE dom_oce         ! ocean space and time domain
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE phycst          ! physical constants
   USE obc_oce         ! ocean open boundary conditions
   USE daymod          ! calendar
   USE in_out_manager  ! I/O logical units
   USE lib_mpp         ! distributed memory computing
   USE dynspg_rl       ! 
   USE ioipsl


#  if ! defined key_dynspg_fsc
   USE obccli
#  endif

   IMPLICIT NONE
   PRIVATE

!! * Accessibility
   PUBLIC obc_dta        ! routines called by step.F90

   !! * Shared module variables
   INTEGER ::   &
      nlecto =  0,    &  ! switch for the first read
      ntobc1      ,   &  ! first record used
      ntobc2             ! second record used
   
   !! * Substitutions
#  include "obc_vectopt_loop_substitute.h90"
   !!---------------------------------------------------------------------------------
   !!   OPA 9.0 , LODYC-IPSL  (2003)
   !!---------------------------------------------------------------------------------

CONTAINS

   SUBROUTINE obc_dta (kt)
      !!--------------------------------------------------------------------
      !!              ***  SUBROUTINE obc_dta  ***
      !!                   
      !! ** Purpose :
      !!   Find the climatological boundary arrays for the specified date,
      !!   The boundary arrays are netcdf files. Three possible cases:
      !!   - one time frame only in the file (time dimension = 1).
      !!     in that case the boundary data does not change in time.
      !!   - many time frames. In that case,  if we have 12 frames
      !!     we assume monthly fields.
      !!     Else, we assume that time_counter is in seconds
      !!     since the beginning of either the current year or a reference
      !!     year given in the namelist.
      !!     (no check is done so far but one would have to check the "unit"
      !!     attribute of variable time_counter).
      !!
      !! History :
      !!        !  98-05 (J.M. Molines) Original code
      !!   8.5  !  02-10 (C. Talandier, A-M. Treguier) Free surface, F90
      !!   9.0  !  04-06 (F. Durand, A-M. Treguier) Netcdf BC files on input
      !!--------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt          ! ocean time-step index
      
      !! * Local declarations
      INTEGER ::   ji, jj, jk, ii, ij   ! dummy loop indices
      INTEGER ::   itimo, iman, imois
      INTEGER ::   i15
      REAL(wp) ::   zxy
      !! * Ajouts FD
      INTEGER ::  iyrel              ! number of years since 1992
      INTEGER ::  isrel              ! number of seconds since 1/1/1992
      INTEGER, SAVE ::  itobce, itobcw,  & ! number of time steps in OBC files
                        itobcs, itobcn,  & !    "       "       "       "
                        itobc
      INTEGER ::  ikprint        ! frequency for printouts.
      INTEGER :: fid_e, fid_w, fid_n, fid_s, fid  ! file identifiers
      LOGICAL :: l_exv
      INTEGER, DIMENSION(flio_max_dims) ::    &
            f_d  ,             & ! dimensions lenght
            start,             & ! starting index read
            count                ! number of indices to be read
      ! time_counter variable of BCs
      REAL(wp),DIMENSION(:),ALLOCATABLE :: ztcobc
     
      CHARACTER(LEN=25) :: f_name,v_name
      !!--------------------------------------------------------------------

      IF( lk_dynspg_rl )  THEN
         CALL obc_dta_psi (kt)     ! update bsf data at open boundaries
         IF( nobc_dta == 1 .AND. kt == nit000 )   THEN
            IF(lwp) WRITE(numout,*) ' time-variable psi boundary data not allowed yet'
            STOP
         ENDIF
      ENDIF
      
      CALL ipslnlf (new_number=numout)
     
      ! 1.   First call: check time frames available in files.
      ! -------------------------------------------------------

      IF( kt == nit000 )   THEN
      
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*)     'obc_dta : find boundary data'
         IF(lwp) WRITE(numout,*)     '~~~~~~~'
             
         IF( nobc_dta == 0 )   THEN
            IF(lwp) WRITE(numout,*)  '  OBC data taken from initial conditions.'
            ntobc1 = 1
            ntobc2 = 1
         ELSE    
            IF(lwp) WRITE(numout,*)  '  OBC data taken from netcdf files.'
            IF(lwp) WRITE(numout,*)  '  climatology (T/F):',ln_obc_clim
            ! check the number of time steps in the files.
            itobce =0 ; itobcw = 0; itobcn = 0; itobcs = 0
            v_name = 'time_counter'
            IF( lp_obc_east )   THEN
               CALL flioopfd ('obceast_TS.nc',fid_e)
               CALL flioinqv (fid_e,TRIM(v_name),l_exv,len_dims=f_d) 
               IF( l_exv )   THEN
                  itobce = f_d(1)
               ELSE
                  WRITE(numout,*) ' Variable ',TRIM(v_name),' not found in file ','obceast_TS.nc'
               ENDIF
            ENDIF
            IF( lp_obc_west )   THEN
               CALL flioopfd ('obcwest_TS.nc',fid_w)
               CALL flioinqv (fid_w,TRIM(v_name),l_exv,len_dims=f_d) 
               IF( l_exv )   THEN
                  itobcw = f_d(1)
               ELSE
                  WRITE(numout,*) ' Variable ',TRIM(v_name),' not found in file ','obcwest_TS.nc'
               ENDIF
            ENDIF
            IF( lp_obc_north )   THEN
               CALL flioopfd ('obcnorth_TS.nc',fid_n)
               CALL flioinqv (fid_n,TRIM(v_name),l_exv,len_dims=f_d) 
               IF( l_exv )   THEN
                  itobcn = f_d(1)
               ELSE
                  WRITE(numout,*) ' Variable ',TRIM(v_name),' not found in file ','obcnorth_TS.nc'
               ENDIF
            ENDIF
            IF( lp_obc_south )   THEN
               CALL flioopfd ('obcsouth_TS.nc',fid_s)
               CALL flioinqv (fid_s,TRIM(v_name),l_exv,len_dims=f_d) 
               IF( l_exv )   THEN
                  itobcs = f_d(1)
               ELSE
                  WRITE(numout,*) ' Variable ',TRIM(v_name),' not found in file ','obcsouth_TS.nc'
               ENDIF
            ENDIF

            itobc = MAX(itobce,itobcw,itobcn,itobcs)
            nstop = 0
            IF( lp_obc_east  .AND. itobce /= itobc ) nstop = nstop+1 
            IF( lp_obc_west  .AND. itobcw /= itobc ) nstop = nstop+1      
            IF( lp_obc_north .AND. itobcn /= itobc ) nstop = nstop+1
            IF( lp_obc_south .AND. itobcs /= itobc ) nstop = nstop+1 
            IF( nstop /= 0 )  THEN
               IF( lwp )   THEN
                  WRITE(numout,*) ' obcdta : all files must have the same number of time steps'
                  WRITE(numout,*) ' east, west, north, south: ', itobce, itobcw, itobcn, itobcs
               ENDIF
               STOP
            ENDIF
            IF( itobc == 1 )   THEN
               IF( lwp ) WRITE(numout,*) ' obcdta found one time step only in the OBC files'
            ELSE
               ALLOCATE (ztcobc(itobc))
               l_exv = .TRUE.
               IF( lp_obc_east )   THEN
                  IF( l_exv )   THEN
                     CALL fliogetv (fid_e,TRIM(v_name),ztcobc)
                     l_exv = .FALSE.
                  ENDIF
                  CALL flioclo (fid_e)
               ENDIF
               IF( lp_obc_west )   THEN
                 IF( l_exv )   THEN
                    CALL fliogetv (fid_w,TRIM(v_name),ztcobc)
                    l_exv = .FALSE.
                 ENDIF
                 CALL flioclo (fid_w)
               ENDIF
               IF( lp_obc_north )   THEN
                 IF( l_exv )   THEN
                    CALL fliogetv (fid_n,TRIM(v_name),ztcobc)
                    l_exv = .FALSE.
                 ENDIF
                 CALL flioclo (fid_n)
               ENDIF
               IF( lp_obc_south )   THEN
                 IF( l_exv )   THEN
                    CALL fliogetv (fid_s,TRIM(v_name),ztcobc)
                    l_exv = .FALSE.
                 ENDIF
                 CALL flioclo (fid_s)
               ENDIF
               IF( lwp ) WRITE(numout,*) ' obcdta found', itobc,' time steps in the OBC files'
               IF( .NOT. ln_obc_clim .AND. itobc == 12 )   THEN
                  IF ( lwp ) WRITE(numout,*) '  WARNING: With monthly data we assume climatology'
                  ln_obc_clim = .true.
               ENDIF
            ENDIF
         ENDIF
      
       ! 1.1  Tangential velocities set to zero
       ! --------------------------------------
         IF( lp_obc_east  ) vfoe = 0.0
         IF( lp_obc_west  ) vfow = 0.0
         IF( lp_obc_south ) ufos = 0.0
         IF( lp_obc_north ) ufon = 0.0
      
       ! 1.2  Data temperature, salinity, normal velocities set to zero
       !                        or initial conditions if nobc_dta == 0
       ! --------------------------------------------------------------

         IF( lp_obc_east )   THEN
            ! initialisation to zero
            sedta(:,:,:) = 0.e0
            tedta(:,:,:) = 0.e0
            uedta(:,:,:) = 0.e0
            !                                    ! ================== !
            IF( nobc_dta == 0 )   THEN           ! initial state used
               !                                 ! ================== !
               !  Fills sedta, tedta, uedta (global arrays)
               !  Remark: this works for njzoom = 1.
               !          Should the definition of ij include njzoom?
               DO ji = nie0, nie1
                  DO jk = 1, jpkm1
                     DO jj = nje0p1, nje1m1
                        ij = jj -1 + njmpp
                        sedta(ij,jk,1) = sn(ji,jj,jk)*tmask(ji,jj,jk)
                        tedta(ij,jk,1) = tn(ji,jj,jk)*tmask(ji,jj,jk)
                        uedta(ij,jk,1) = un(ji,jj,jk)*umask(ji,jj,jk)
                     END DO
                  END DO
               END DO
            ENDIF
         ENDIF

         IF( lp_obc_west )   THEN
            ! initialisation to zero
            swdta(:,:,:) = 0.e0
            twdta(:,:,:) = 0.e0
            uwdta(:,:,:) = 0.e0
            !                                    ! ================== !
            IF( nobc_dta == 0 )   THEN           ! initial state used !
               !                                 ! ================== !
               !  Fills swdta, twdta, uwdta (global arrays)
               !  Remark: this works for njzoom = 1. 
               !          Should the definition of ij include njzoom?
               DO ji = niw0, niw1
                  DO jk = 1, jpkm1
                     DO jj = njw0p1, njw1m1
                        ij = jj -1 + njmpp
                        swdta(ij,jk,1) = sn(ji,jj,jk)*tmask(ji,jj,jk)
                        twdta(ij,jk,1) = tn(ji,jj,jk)*tmask(ji,jj,jk)
                        uwdta(ij,jk,1) = un(ji,jj,jk)*umask(ji,jj,jk)
                     END DO
                  END DO
               END DO
            ENDIF
         ENDIF

         IF( lp_obc_north)   THEN
            ! initialisation to zero
            sndta(:,:,:) = 0.e0
            tndta(:,:,:) = 0.e0
            vndta(:,:,:) = 0.e0
            !                                    ! ================== !
            IF( nobc_dta == 0 )   THEN           ! initial state used
               !                                 ! ================== !
               !  Fills sndta, tndta, vndta (global arrays)
               !  Remark: this works for njzoom = 1. 
               !          Should the definition of ij include njzoom?
               DO jj = njn0, njn1
                  DO jk = 1, jpkm1
                     DO ji = nin0p1, nin1m1
                        ii = ji -1 + nimpp
                        sndta(ii,jk,1) = sn(ji,jj,jk)*tmask(ji,jj,jk)
                        tndta(ii,jk,1) = tn(ji,jj,jk)*tmask(ji,jj,jk)
                        vndta(ii,jk,1) = vn(ji,jj,jk)*vmask(ji,jj,jk)
                     END DO
                  END DO
               END DO
            ENDIF
         ENDIF

         IF( lp_obc_south )   THEN
            ! initialisation to zero
            ssdta(:,:,:) = 0.e0
            tsdta(:,:,:) = 0.e0
            vsdta(:,:,:) = 0.e0
            !                                    ! ================== !
            IF( nobc_dta == 0 )   THEN           ! initial state used
               !                                 ! ================== !
               !  Fills ssdta, tsdta, vsdta (global arrays)
               !  Remark: this works for njzoom = 1. 
               !          Should the definition of ij include njzoom?
               DO jj = njs0, njs1
                  DO jk = 1, jpkm1
                     DO ji = nis0p1, nis1m1
                        ii = ji -1 + nimpp
                        ssdta(ii,jk,1) = sn(ji,jj,jk)*tmask(ji,jj,jk)
                        tsdta(ii,jk,1) = tn(ji,jj,jk)*tmask(ji,jj,jk)
                        vsdta(ii,jk,1) = vn(ji,jj,jk)*vmask(ji,jj,jk)
                     END DO
                  END DO
               END DO
            ENDIF
         ENDIF

      ENDIF        !       end if kt == nit000
      
      ! 2.  Initialize the time we are at.
      !     Does this every time the routine is called,
      !     excepted when nobc_dta = 0
      !---------------------------------------------------------------------
      IF( nobc_dta == 0 )   THEN
         itimo = 1
         zxy   = 0
      ELSE
         IF( itobc == 1 )   THEN
            itimo = 1
         ELSE IF( itobc == 12 )   THEN      !   BC are monthly   
            ! we assume we have climatology in that case
            iman  = 12
            i15   = nday / 16
            imois = nmonth + i15 - 1
            IF( imois == 0 )   imois = iman
            itimo = imois   
         ELSE
            IF(lwp) WRITE(numout,*) 'data other than constant or monthly not written yet'
            STOP
         ENDIF
      ENDIF
      
      ! 2.1 Read two records in the file if necessary
      ! ---------------------------------------------
      IF( ( nobc_dta == 1 ) .AND. ( ( kt == nit000 .AND. nlecto == 0 ) .OR. itimo  /= ntobc1 ) )   THEN
         nlecto = 1
      
         ! Calendar computation
         IF( itobc == 1 )   THEN            !  BC are constant in time
            ntobc1 = 1
            ntobc2 = 1  
         ELSE IF( itobc == 12 )   THEN      !   BC are monthly   
            ntobc1 = itimo         ! first file record used
            ntobc2 = ntobc1 + 1    ! last  file record used
            ntobc1 = MOD( ntobc1, iman )
            IF( ntobc1 == 0 )   ntobc1 = iman
            ntobc2 = MOD( ntobc2, iman )
            IF( ntobc2 == 0 )   ntobc2 = iman
            IF( lwp )   THEN
               WRITE(numout,*) ' read monthly obc first record file used ntobc1 ', ntobc1
               WRITE(numout,*) ' read monthly obc last  record file used ntobc2 ', ntobc2
            ENDIF
         ELSE
            !!!!!!!!!!!!!ATTENTION el: A verifier en fction de la convention choisie pour
            !!!!!!!!!!!!! le codage de nyear, pour les runs interannuels !!!!!!!!!!!!!!
            !!!  attention if ln_obc_clim is true, go back to jan 1st after december 31st
            iyrel=nyear-1991
            IF( ( iyrel < 1 ) .OR. ( iyrel > 13 ) )   THEN
               IF( lwp ) WRITE(numout,*) 'Pb OBCDTA : iyrel'
               STOP
            ENDIF
            ! Compute nb of seconds from 1/1/1992 00:00 :
            isrel=(365*(iyrel-1)+nday_year)*86400
            IF( lwp )   THEN
               WRITE(numout,*)'Nbre de secondes ecoulees depuis le 1/1/1992:'
               WRITE(numout,*) isrel
            ENDIF
      
            !  need to calculate here ntobc1 and ntobc2, the two time steps to be read
      
         ENDIF
                               ! ======================= !
                               !  BCs read               !
         !                     ! ======================= !
         IF( lp_obc_east )   THEN
            ! ... Read datafile and set temperature, salinity and normal velocity
            ! ... initialise the sedta, tedta, uedta arrays
            CALL flioopfd ('obceast_TS.nc',fid_e)
            CALL obc_dta_gv (fid_e,'y','vosaline',sedta(:,:,1),jpjef-jpjed+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_e,'y','vosaline',sedta(:,:,2),jpjef-jpjed+1,jpk,ntobc2)
            CALL obc_dta_gv (fid_e,'y','votemper',tedta(:,:,1),jpjef-jpjed+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_e,'y','votemper',tedta(:,:,2),jpjef-jpjed+1,jpk,ntobc2)
            CALL flioclo (fid_e)

            CALL flioopfd ('obceast_U.nc',fid_e)
            CALL obc_dta_gv (fid_e,'y','vozocrtx',uedta(:,:,1),jpjef-jpjed+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_e,'y','vozocrtx',uedta(:,:,2),jpjef-jpjed+1,jpk,ntobc2)
            CALL flioclo (fid_e)
            !  Usually printout is done only once at kt = nit000,
            !  unless nprint (namelist) > 1      
            IF( lwp .AND.  ( kt == nit000 .OR. nprint /= 0 ) )   THEN
               WRITE(numout,*)
               WRITE(numout,*) ' Read East OBC data records ', ntobc1, ntobc2
               ikprint = (jpjef-jpjed+1)/20 +1
               WRITE(numout,*) ' Temperature record 1 - printout every 3 level'
               CALL prihre( tedta(:,:,1),jpjef-jpjed+1,jpk,1,jpjef-jpjed+1,ikprint, &
                  &         jpk, 1, -3, 1., numout )
               WRITE(numout,*)
               WRITE(numout,*) ' Salinity  record 1 - printout every 3 level'
               CALL prihre( sedta(:,:,1), jpjef-jpjed+1, jpk, 1, jpjef-jpjed+1, ikprint, &
                  &        jpk, 1, -3, 1., numout )
               WRITE(numout,*)
               WRITE(numout,*) ' Normal velocity U  record 1  - printout every 3 level'
               CALL prihre( uedta(:,:,1), jpjef-jpjed+1, jpk, 1, jpjef-jpjed+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
            ENDIF
         ENDIF

         IF( lp_obc_west )   THEN
            ! ... Read datafile and set temperature, salinity and normal velocity
            ! ... initialise the swdta, twdta, uwdta arrays
            CALL flioopfd ('obcwest_TS.nc',fid_w)
            CALL obc_dta_gv (fid_w,'y','vosaline',swdta(:,:,1),jpjwf-jpjwd+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_w,'y','vosaline',swdta(:,:,2),jpjwf-jpjwd+1,jpk,ntobc2)
            CALL obc_dta_gv (fid_w,'y','votemper',twdta(:,:,1),jpjwf-jpjwd+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_w,'y','votemper',twdta(:,:,2),jpjwf-jpjwd+1,jpk,ntobc2)
            CALL flioclo (fid_w)

            CALL flioopfd ('obcwest_U.nc',fid_w)
            CALL obc_dta_gv (fid_w,'y','vozocrtx',uwdta(:,:,1),jpjwf-jpjwd+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_w,'y','vozocrtx',uwdta(:,:,2),jpjwf-jpjwd+1,jpk,ntobc2)
            CALL flioclo (fid_w)

            IF( lwp .AND.  ( kt == nit000 .OR. nprint /= 0 ) )   THEN
               WRITE(numout,*)
               WRITE(numout,*) ' Read West OBC data records ', ntobc1, ntobc2
               ikprint = (jpjwf-jpjwd+1)/20 +1
               WRITE(numout,*) ' Temperature  record 1 - printout every 3 level'
               CALL prihre( twdta(:,:,1), jpjwf-jpjwd+1, jpk, 1, jpjwf-jpjwd+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
               WRITE(numout,*)
               WRITE(numout,*) ' Salinity  record 1 - printout every 3 level'
               CALL prihre( swdta(:,:,1), jpjwf-jpjwd+1, jpk, 1, jpjwf-jpjwd+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
               WRITE(numout,*)
               WRITE(numout,*) ' Normal velocity U  record 1  - printout every 3 level'
               CALL prihre( uwdta(:,:,1), jpjwf-jpjwd+1, jpk, 1, jpjwf-jpjwd+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
            ENDIF
         ENDIF

         IF( lp_obc_north )   THEN      
            CALL flioopfd ('obcnorth_TS.nc',fid_n)
            CALL obc_dta_gv (fid_n,'x','vosaline',sndta(:,:,1),jpinf-jpind+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_n,'x','vosaline',sndta(:,:,2),jpinf-jpind+1,jpk,ntobc2)
            CALL obc_dta_gv (fid_n,'x','votemper',tndta(:,:,1),jpinf-jpind+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_n,'x','votemper',tndta(:,:,2),jpinf-jpind+1,jpk,ntobc2)
            CALL flioclo (fid_n)

            CALL flioopfd ('obcnorth_V.nc',fid_n)
            CALL obc_dta_gv (fid_n,'x','vomecrty',vndta(:,:,1),jpinf-jpind+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_n,'x','vomecrty',vndta(:,:,2),jpinf-jpind+1,jpk,ntobc2)
            CALL flioclo (fid_n)

            IF( lwp .AND.  ( kt == nit000 .OR. nprint /= 0 ) )   THEN
               WRITE(numout,*)
               WRITE(numout,*) ' Read North OBC data records ', ntobc1, ntobc2
               ikprint = (jpinf-jpind+1)/20 +1
               WRITE(numout,*) ' Temperature  record 1 - printout every 3 level'
               CALL prihre( tndta(:,:,1), jpinf-jpind+1, jpk, 1, jpinf-jpind+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
               WRITE(numout,*)
               WRITE(numout,*) ' Salinity  record 1 - printout every 3 level'
               CALL prihre( sndta(:,:,1), jpinf-jpind+1, jpk, 1, jpinf-jpind+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
               WRITE(numout,*)
               WRITE(numout,*) ' Normal velocity V  record 1  - printout every 3 level'
               CALL prihre( vndta(:,:,1), jpinf-jpind+1, jpk, 1, jpinf-jpind+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
            ENDIF
         ENDIF

         IF( lp_obc_south )   THEN      
            CALL flioopfd ('obcsouth_TS.nc',fid_s)
            CALL obc_dta_gv (fid_s,'x','vosaline',ssdta(:,:,1),jpisf-jpisd+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_s,'x','vosaline',ssdta(:,:,2),jpisf-jpisd+1,jpk,ntobc2)
            CALL obc_dta_gv (fid_s,'x','votemper',tsdta(:,:,1),jpisf-jpisd+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_s,'x','votemper',tsdta(:,:,2),jpisf-jpisd+1,jpk,ntobc2)
            CALL flioclo (fid_s)

            CALL flioopfd ('obcsouth_V.nc',fid_s)
            CALL obc_dta_gv (fid_s,'x','vomecrty',vsdta(:,:,1),jpisf-jpisd+1,jpk,ntobc1)
            CALL obc_dta_gv (fid_s,'x','vomecrty',vsdta(:,:,2),jpisf-jpisd+1,jpk,ntobc2)
            CALL flioclo (fid_s)

            IF( lwp .AND.  ( kt == nit000 .OR. nprint /= 0 ) )   THEN
               WRITE(numout,*)
               WRITE(numout,*) ' Read South OBC data records ', ntobc1, ntobc2
               ikprint = (jpisf-jpisd+1)/20 +1
               WRITE(numout,*) ' Temperature  record 1 - printout every 3 level'
               CALL prihre( tsdta(:,:,1), jpisf-jpisd+1, jpk, 1, jpisf-jpisd+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
               WRITE(numout,*)
               WRITE(numout,*) ' Salinity  record 1 - printout every 3 level'
               CALL prihre( ssdta(:,:,1), jpisf-jpisd+1, jpk, 1, jpisf-jpisd+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
               WRITE(numout,*)
               WRITE(numout,*) ' Normal velocity V  record 1  - printout every 3 level'
               CALL prihre( vsdta(:,:,1), jpisf-jpisd+1, jpk, 1, jpisf-jpisd+1, ikprint, &
                  &         jpk, 1, -3, 1., numout )
            ENDIF
         ENDIF
      
      ENDIF        !      end of the test on the condition to read or not the files 
      
      ! 3.  Call at every time step :
      !     Linear interpolation of BCs to current time step 
      ! ----------------------------------------------------

      IF( itobc == 1 .OR. nobc_dta == 0 )   THEN 
         zxy = 0.
      ELSE IF( itobc == 12 )   THEN         
         zxy = FLOAT( nday + 15 - 30 * i15 ) / 30.
      ELSE
         zxy = (ztcobc(ntobc1)-FLOAT(isrel))/(ztcobc(ntobc1)-ztcobc(ntobc2))
      ENDIF
      
      IF( lp_obc_east )   THEN
         !  fills sfoe, tfoe, ufoe (local to each processor)
         DO jk = 1, jpkm1
            DO jj = nje0p1, nje1m1
               ij = jj -1 + njmpp
               sfoe(jj,jk) =  ( zxy * sedta(ij,jk,2) + &
                  &           (1.-zxy) * sedta(ij,jk,1) ) * temsk(jj,jk)
               tfoe(jj,jk) =  ( zxy * tedta(ij,jk,2) + &
                  &           (1.-zxy) * tedta(ij,jk,1) ) * temsk(jj,jk)
               ufoe(jj,jk) =  ( zxy * uedta(ij,jk,2) + &
                  &           (1.-zxy) * uedta(ij,jk,1) ) * uemsk(jj,jk)
            END DO
         END DO
      ENDIF
      
      IF( lp_obc_west )   THEN
         !  fills sfow, tfow, ufow (local to each processor)
         DO jk = 1, jpkm1
            DO jj = njw0p1, njw1m1
               ij = jj -1 + njmpp
               sfow(jj,jk) =  ( zxy * swdta(ij,jk,2) + &
                  &           (1.-zxy) * swdta(ij,jk,1) ) * twmsk(jj,jk)
               tfow(jj,jk) =  ( zxy * twdta(ij,jk,2) + &
                  &           (1.-zxy) * twdta(ij,jk,1) ) * twmsk(jj,jk)
               ufow(jj,jk) =  ( zxy * uwdta(ij,jk,2) + &
                  &           (1.-zxy) * uwdta(ij,jk,1) ) * uwmsk(jj,jk)
            END DO
         END DO
      ENDIF
      
      IF( lp_obc_north )   THEN
         !  fills sfon, tfon, vfon (local to each processor)
         DO jk = 1, jpkm1
            DO ji = nin0p1, nin1m1
               ii = ji -1 + nimpp
               sfon(ji,jk) =  ( zxy * sndta(ii,jk,2) + &
                  &           (1.-zxy) * sndta(ii,jk,1) ) * tnmsk(ji,jk)
               tfon(ji,jk) =  ( zxy * tndta(ii,jk,2) + &
                  &           (1.-zxy) * tndta(ii,jk,1) ) * tnmsk(ji,jk)
               vfon(ji,jk) =  ( zxy * vndta(ii,jk,2) + &
                  &           (1.-zxy) * vndta(ii,jk,1) ) * vnmsk(ji,jk)
            END DO
         END DO
      ENDIF
      
      IF( lp_obc_south )   THEN
         !  fills sfos, tfos, vfos (local to each processor)
         DO jk = 1, jpkm1
           DO ji = nis0p1, nis1m1
              ii = ji -1 + nimpp
              sfos(ji,jk) = ( zxy * ssdta(ii,jk,2) + &
                 &          (1.-zxy) * ssdta(ii,jk,1) ) * tsmsk(ji,jk)
              tfos(ji,jk) = ( zxy * tsdta(ii,jk,2) + &
                 &          (1.-zxy) * tsdta(ii,jk,1) ) * tsmsk(ji,jk)
              vfos(ji,jk) = ( zxy * vsdta(ii,jk,2) + &
                 &          (1.-zxy) * vsdta(ii,jk,1) ) * vsmsk(ji,jk)
           END DO
         END DO             
      ENDIF
      
   END SUBROUTINE obc_dta
      
# if defined key_dynspg_fsc
   !!-----------------------------------------------------------------------------
   !!   'key_dynspg_fsc'                    free surface with constant volume
   !!-----------------------------------------------------------------------------
   SUBROUTINE obc_dta_psi ( kt )       ! Empty routine
      !! * Arguments
      INTEGER,INTENT(in) :: kt 
      WRITE(*,*) 'obc_dta_psi: You should not have seen this print! error?', kt
   END SUBROUTINE obc_dta_psi
#else
   !!-----------------------------------------------------------------------------
   !!   Default option                                                   Rigid-lid
   !!-----------------------------------------------------------------------------

   SUBROUTINE obc_dta_psi ( kt )
      !!-----------------------------------------------------------------------------
      !!                       ***  SUBROUTINE obc_dta_psi  ***
      !!
      !! ** Purpose :
      !!      Update the climatological streamfunction OBC at each time step.
      !!      Depends on the user's configuration.  Here data are read only once
      !!      at the beginning of the run.
      !!
      !! ** Method :
      !!      1. initialization
      !!         kbsfstart: number of time steps over which increase bsf
      !!         during initialization. This is provided for a smooth start
      !!         in cases where the transport is large (like on the Antarctic
      !!         continent). also note that when kbfstart=1, the transport
      !!         increases a lot in one time step and the precision usually
      !!         required for the solver may not be enough.
      !!      2. set the time evolution of the climatological barotropic streamfunction
      !!         along the isolated coastlines ( gcbic(jnic) ). 
      !!      3. set the climatological barotropic streamfunction at the boundary.
      !!
      !!      The last two steps are done only at first step (nit000) or if kt <= kbfstart
      !!
      !! History :
      !!        ! 97-08 (G. Madec, J.M. Molines)
      !!   8.5  ! 02-10 (C. Talandier, A-M. Treguier) Free surface, F90
      !!----------------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt          ! ocean time-step index

      !! * Local declarations
      INTEGER ::   ji, jj, jnic, jip         ! dummy loop indices
      INTEGER ::   inum = 11                 ! temporary logical unit
      INTEGER ::   ip, ii, ij, iii, ijj
      INTEGER ::   kbsfstart
      REAL(wp) ::   zsver1, zsver2, zsver3, z2dtr, zcoef
      !!----------------------------------------------------------------------------

      ! 1. initialisation
      ! -----------------

      kbsfstart =  1
      zsver1 =  bsfic0(1)
      zsver2 =  zsver1
      IF( kt <= kbsfstart )   THEN
         zcoef = float(kt)/float(kbsfstart)
      ELSE
         zcoef = 1.
      END IF
      bsfic(1) = zsver1*zcoef
      IF( lwp .AND. ( kt <= kbsfstart ) )   THEN
         IF(lwp) WRITE(numout,*)'            '
         IF(lwp) WRITE(numout,*)'obcdta: spinup phase in obc_dta_psi routine'
         IF(lwp) WRITE(numout,*)'~~~~~~  it=',kt,'  OBC: spinup coef: ', &
                                           zcoef, ' and transport: ',bsfic(1)
      END IF

      zsver2 =  bsfic(1)-bsfic(2)
      zsver3 =  bsfic(2)

      ! 2. Right hand side of the barotropic elliptic equation (isolated coastlines)
      ! ----------------------------------------------------------------------------

      IF( ( neuler == 0 ) .AND. ( kt == nit000 ) )   THEN
         z2dtr = 1./rdt
      ELSE
         z2dtr = 1./2./rdt
      END IF
      ! ... bsfb(ii,ij) should be constant but due to the Asselin filter it
      ! ... converges asymptotically towards bsfic(jnic)
      ! ... However, bsfb(ii,ij) is constant along the same coastlines
      ! ... ---> can be improved using an extra array for storing bsficb (before)
      IF( nbobc > 1 )   THEN
         DO jnic = 1,nbobc - 1
            gcbic(jnic) = 0.
            ip=mnic(0,jnic)
            DO jip = 1,ip
               ii = miic(jip,0,jnic) 
               ij = mjic(jip,0,jnic) 
               IF( ii >= nldi+ nimpp - 1 .AND. ii <= nlei+ nimpp - 1 .AND. &
                   ij >= nldj+ njmpp - 1 .AND. ij <= nlej+ njmpp - 1 )   THEN
                  iii=ii-nimpp+1
                  ijj=ij-njmpp+1
                  gcbic(jnic) = ( bsfic(jnic) - bsfb(iii,ijj) ) * z2dtr
               END IF
            END DO
         END DO
      END IF

      IF( lk_mpp )   CALL mpp_isl( gcbic, 3 )

      ! 3. Update the climatological barotropic function at the boundary 
      ! ----------------------------------------------------------------

      IF( lpeastobc )   THEN

         IF( kt == nit000 .OR. kt <= kbsfstart )   THEN
            OPEN(inum,file='obceastbsf.dta')
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*) (bfoe(jj),jj=jpjed, jpjef)
            CLOSE(inum)
         END IF
         DO jj=jpjed, jpjefm1
            bfoe(jj)=bfoe(jj)*zcoef
         END DO

      END IF

      IF( lpwestobc)   THEN

         IF( kt == nit000 .OR. kt <= kbsfstart ) then
            OPEN(inum,file='obcwestbsf.dta')
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*) (bfow(jj),jj=jpjwd, jpjwf)
            CLOSE(inum)
         END IF
         DO jj=jpjwd, jpjwfm1
            bfow(jj)=bfow(jj)*zcoef
         END DO

      END IF

      IF( lpsouthobc)   THEN

         IF( kt == nit000.OR.kt <= kbsfstart )   THEN
            OPEN(inum,file='obcsouthbsf.dta')
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*) (bfos(jj),jj=jpisd, jpisf)
            CLOSE(inum)
         END IF
         DO ji=jpisd, jpisfm1
            bfos(ji)=bfos(ji)*zcoef
         END DO

      END IF

      IF( lpnorthobc)   THEN
         IF( kt == nit000.OR.kt <= kbsfstart )   THEN
            OPEN(inum,file='obcnorthbsf.dta')
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*)
            READ(inum,*) (bfon(jj),jj=jpind, jpinf)
            CLOSE(inum)
         END IF
         DO ji=jpind, jpinfm1
            bfon(ji)=bfon(ji)*zcoef
         END DO

      END IF

   END SUBROUTINE obc_dta_psi

# endif

   SUBROUTINE obc_dta_gv (ifid,cldim,clobc,pdta,kobcij,kobck,ktobc)
      !!-----------------------------------------------------------------------------
      !!                       ***  SUBROUTINE obc_dta_gv  ***
      !!
      !! ** Purpose :   Read a OBC forcing field from netcdf file 
      !!                Input file are supposed to be 3D e.g.
      !!                - for a South or North OB : longitude x depth x time
      !!		- for a West or East OB : latitude x depth x time
      !!
      !! History :
      !!        !  04-06 (A.-M. Treguier, F. Durand) Original code
      !!        !  05-02 (J. Bellier, C. Talandier) use fliocom CALL
      !!----------------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT(IN) ::   &
         ifid,                 & ! netcdf file name identifier
         kobcij,               & ! Horizontal (i or j) dimension of the array
         kobck,                & ! vertical dimension
         ktobc                   ! starting time index read
      CHARACTER(LEN=*), INTENT(IN)    ::   &
         cldim,                & ! dimension along which is the open boundary ('x' or 'y')
         clobc                   ! name of the netcdf variable read
      REAL, DIMENSION(kobcij,kobck,1), INTENT(OUT) ::   &
         pdta                    ! 3D array of OBC forcing field
      
      !! * Local declarations
      INTEGER ::   indim
      LOGICAL ::   l_exv
      INTEGER,DIMENSION(4) ::   f_d, istart, icount
      REAL(wp), DIMENSION(:,:,:,:), ALLOCATABLE ::   v_tmp_4
      !----------------------------------------------------------------------

      CALL flioinqv (ifid,TRIM(clobc),l_exv,nb_dims=indim,len_dims=f_d) 
      IF( l_exv )   THEN
         ! checks the number of dimensions
         IF( indim == 3 )   THEN
            istart(1:3) = (/ 1, 1, ktobc /)
            icount(1:3) = (/ kobcij, kobck, 1 /)
            CALL fliogetv (ifid,TRIM(clobc),pdta,start=istart(1:3),count=icount(1:3))
         ELSE IF( indim == 4 )   THEN
            istart(1:4) = (/ 1, 1, 1, ktobc /)
            IF( TRIM(cldim) == 'y' )   THEN
               icount(1:4) = (/ 1, kobcij, kobck, 1 /)
            ELSE
               icount(1:4) = (/ kobcij, 1, kobck, 1 /)
            ENDIF
            ALLOCATE (v_tmp_4(icount(1),icount(2),icount(3),icount(4)))
            CALL fliogetv (ifid,TRIM(clobc),v_tmp_4,start=istart(1:4),count=icount(1:4))
            IF( TRIM(cldim) == 'y' )   THEN
               pdta(1:kobcij,1:kobck,1:1) = v_tmp_4(1,1:kobcij,1:kobck,1:1)
            ELSE
               pdta(1:kobcij,1:kobck,1:1) = v_tmp_4(1:kobcij,1,1:kobck,1:1)
            ENDIF
            DEALLOCATE (v_tmp_4)
         ELSE
            IF( lwp )   THEN
               WRITE(numout,*) ' Problem in OBC file for ',TRIM(clobc),' :'
               WRITE(numout,*) ' number of dimensions (not 3 or 4) =',indim
            ENDIF
            STOP
         ENDIF
      ELSE
         WRITE(numout,*) ' Variable ',TRIM(clobc),' not found'
      ENDIF
      
   END SUBROUTINE obc_dta_gv

#else
   !!--------------------------------------------------------------------
   !!  default option  :  Dummy module    NO Open Boundary Conditions
   !!--------------------------------------------------------------------
CONTAINS
   SUBROUTINE obc_dta( kt )             ! Dummy routine
     INTEGER, INTENT (in) :: kt
     WRITE(*,*) 'obc_dta: You should not have seen this print! error?', kt
   END SUBROUTINE obc_dta
#endif

   !!=====================================================================
END MODULE obcdta
